/*-----------------------------------------------------------------------
_dreams_run.do

Stata do file to perform data construction and analysis for
'City of dreams',
by Jorge De la Roca, Gianmarco Ottaviano and Diego Puga,
Journal of the European Economic Association
*
* confidential files needed:
* data/src/nlsy/geocode/location-value-labels.do,
* data/src/nlsy/geocode/survey_and_created_variables-value-labels.do
* data/src/nlsy/geocode/location.dct
* data/src/nlsy/geocode/survey_and_created_variables.dct
-----------------------------------------------------------------------*/

* set all localisations here --------------------------------------------
* when running under Windows
if "`c(os)'" == "Windows" {
	* path to root of project tree
	global PathProjectRoot "D:/dreams/replication"
	}
* when running under MacOS/Unix
else {
	* path to root of project tree
	global PathProjectRoot "/Users/Archivos/dreams/replication"
}
*------------------------------------------------------------------------

* open log
capture mkdir "$PathProjectRoot/code/logs"
capture log close
local datetime : di %tcCCYY.NN.DD!_HH.MM.SS `=clock("$S_DATE $S_TIME", "DMYhms")'
local logfile "$PathProjectRoot/code/logs/log_`datetime'.txt"
log using "`logfile'", text
timer clear
di "Stata version: `c(stata_version)' `=cond( c(MP),"MP `c(processors)'",cond(c(SE),"SE",c(flavor)) )'"
di "Update level:  `c(born_date)'"
di "OS:            `c(os)' `c(osdtl)'"

* set flags here --------------------------------------------------------
* to install the required stata packages, set the following flag to 1
global InstallPackages = 0
* to disable the part of the data construction that requires the 
* restricted-access geocode nlsy79 data, set the following flag to 1;
* note that, without the restricted-access geocode nlsy79 data,
* no location data will be available, and it will not be possible to 
* replicate our results; however, the nlñsy79 panel generated may be 
* useful for other projects;
* enabling the part of the data construction that requires the 
* restricted-access geocode nlsy79 data, involves the files:
* data/src/nlsy/geocode/location-value-labels.do,
* data/src/nlsy/geocode/survey_and_created_variables-value-labels.do
* data/src/nlsy/geocode/location.dct
* data/src/nlsy/geocode/survey_and_created_variables.dct
global NLSYDisableGeocode = 0
* users without access to the geocode nlsy79 data that wish to check
* that the replication code runs smoothly can set the following flag to 1;
* this will randomly generate a fake location history for each respondent
* that will allow the code to run but generate meaningless results with 
* the same format but different values than the actual results in the article
global NLSYGenerateFakeLocations = 0
*------------------------------------------------------------------------

* flag consistency check
if ("$NLSYGenerateFakeLocations" == "1") {
	global NLSYDisableGeocode = 1
}

* initialise
clear all
set more off
set varabbrev off

* stata version control
* make newer stata versions behave like 17
if (`c(stata_version)' > 17) {
	version 17
}
* version 14 or higher required for unicode support
else {
	version 14
}

* set random-number seed
set seed 954728

* optionally install required stata packages
if ("$InstallPackages" == "1") {
	net install st0085_2.pkg, from(http://www.stata-journal.com/software/sj14-2) replace
	ssc install grstyle, replace
}
display "Installed versions of required Stata packages:"
ado describe estout
ado describe grstyle
	
* create directories for output files if needed
capture mkdir "$PathProjectRoot/data/processed"
capture mkdir "$PathProjectRoot/results"

* graphics settings
grstyle clear
set scheme s2color
grstyle init
grstyle color background     white
grstyle linestyle plotregion none
grstyle linestyle grid       none
grstyle linestyle major_grid none
grstyle linestyle minor_grid none
grstyle set graphsize 14cm 14cm
grstyle set size 11pt: axis_title body
grstyle set size 8pt: subheading
grstyle set margin tiny: subheading
grstyle set size 8pt: tick_label
grstyle anglestyle vertical_tick vertical
grstyle textboxstyle leg_caption subheading
grstyle yesno legend_force_nodraw yes
graph set eps logo off
graph set eps fontface Palatino

* set population threshold for big city
global size_big = 2000000

* construct datasets from raw sources
cd "$PathProjectRoot/code"
timer on 1
do 1_dreams_builddata.do
timer off 1

* perform analysis
timer on 2
* only if geocode data is available or if fake location data has been generated 
if ("$NLSYDisableGeocode" != "1" | "$NLSYGenerateFakeLocations" == "1") {
	do 2_dreams_analysis.do
}
timer off 2

* report timers
qui timer list
local timer_1 = round(`r(t1)')
local timer_2 = round(`r(t2)')
di "Time taken by data construction: `timer_1' seconds"
di "Time taken by analysis:          `timer_2' seconds"

* end log
clear
log close
