/*-----------------------------------------------------------------------
dreams_figure2.do

Stata do file to replicate figure 2 of the article
'City of dreams',
by Jorge De la Roca, Gianmarco Ottaviano and Diego Puga,
Journal of the European Economic Association
-----------------------------------------------------------------------*/

* initialise
clear
set more off
set varabbrev off

local groups 3

tempfile aggregates
tempfile grid36
tempfile tmp_self
tempfile grid64

use ../../data/processed/nlsy_panel_sample, clear

* Assign zero values of population to non-metro areas
replace cbsa_pop2010 = 0 if !missing(fips) & missing(cbsa_pop2010)
* Keep only junior and senior periods
keep if junior_period == 1 | senior_period == 1
* Keep individuals observed in both periods
egen count = seq(), by(person_id)
egen max = max(count), by (person_id)
keep if max == 2
keep person_id afqt_pctl self_pctl cbsa_pop2010 count
* Reshape to individual-level data
reshape wide cbsa_pop2010, i(person_id) j(count)
rename (cbsa_pop20101 cbsa_pop20102) (cbsa_pop_junior cbsa_pop_senior)
* Construct CBSA sizes in both periods
gen sma_x = 1 if cbsa_pop_junior < $size_big
replace sma_x = 0 if missing(sma_x)
gen big_x = 1 if cbsa_pop_junior >= $size_big & !missing(cbsa_pop_junior)
replace big_x = 0 if missing(big_x)
gen x_sma = 1 if cbsa_pop_senior < $size_big
replace x_sma = 0 if missing(x_sma)
gen x_big = 1 if cbsa_pop_senior >= $size_big & !missing(cbsa_pop_senior)
replace x_big = 0 if missing(x_big)
* Construct four possible location scenarios
gen s_s = 1 if sma_x == 1 & x_sma == 1
gen s_b = 1 if sma_x == 1 & x_big == 1
gen b_s = 1 if big_x == 1 & x_sma == 1
gen b_b = 1 if big_x == 1 & x_big == 1
gen people = 1
* Create terciles of ability and self-confidence
egen qt_abil = cut(afqt_pctl), group(`groups') icodes
replace qt_abil = qt_abil + 1
egen qt_self = cut(self_pctl), group(`groups') icodes
replace qt_self = qt_self + 1

** figure 2a: Bivariate distribution of self-confidence and ability
* Calculate aggregates of location choices by ability and self-confidence
collapse (sum) s_s s_b b_s b_b people, by(qt_abil qt_self)
* Calculate shares of individuals by cell
gen pos = 12
egen sum_people = sum(people)
gen share_people = people / sum_people * 100
replace share_people = round(share_people,0.01)
format share_people %9.1f
twoway (scatter qt_abil qt_self, mcolor(none) msize(zero) mlabel(share_people) mlabsize(large) mlabcolor(dknavy) mlabposition(0)), xline(1.5 2.5, lcolor(black) lpattern(dash) lwidth(thin)) yline(1.5 2.5, lcolor(black) lpattern(dash) lwidth(thin)) xline(3.5, lcolor(black)  lwidth(thin)) yline(3.5, lcolor(black)  lwidth(thin)) yscale(range(0.5 3.5) extend) xscale(range(0.5 3.5) extend) /*
*/ xlabel(1 2 3, format(%12.0fc) tstyle(notick)) ylabel(1 2 3, format(%12.0fc) tstyle(notick)) ytitle("Ability tercile") xtitle("Self-confidence tercile") plotregion(margin(zero))
graph export ../../results/dreams_fig2a.png, as(png) replace
graph export ../../results/dreams_fig2a.eps, as(eps) replace

** figure 2b: Prevalent location choice
* Calculate aggregates of location choices by ability and self-confidence
foreach var of varlist s_s - b_b {
	gen sh_`var' = `var' / people
	egen sum_`var' = sum(`var')
	gen sh_`var'_avg = sum_`var' / sum_people
	gen sh_`var'_rel2avg = sh_`var' - sh_`var'_avg
	replace sh_`var' = sh_`var'_rel2avg
}
save "`aggregates'", replace
keep qt_abil qt_self sh_s_s sh_s_b sh_b_s sh_b_b pos
egen sh_max = rmax(sh*)
gen domin = .
replace domin = 1 if sh_s_s == sh_max
replace domin = 2 if sh_s_b == sh_max
replace domin = 3 if sh_b_s == sh_max
replace domin = 4 if sh_b_b == sh_max
label define domin 1 "SS" 2 "SB" 3 "BS" 4 "BB"
label val domin domin
twoway (scatter qt_abil qt_self, mcolor(none) msize(zero) mlabel(domin) mlabsize(large) mlabcolor(dknavy) mlabv(pos) mlabposition(0)), xline(1.5, lcolor(black) lpattern(dash) lwidth(thin)) yline(1.5 2.5, lcolor(black) lpattern(dash) lwidth(thin)) xline(2.5, lcolor(black) lpattern(dash) lwidth(thin)) xline(3.5, lcolor(black) lwidth(thin)) yline(3.5, lcolor(black) lwidth(thin)) yscale(range(0.5 3.5) extend) xscale(range(0.5 3.5) extend) xlabel(1 2 3, format(%12.0fc) tstyle(notick)) ylabel(1 2 3, format(%12.0fc) tstyle(notick)) ytitle("Ability tercile") xtitle("Self-confidence tercile") plotregion(margin(zero))
graph export ../../results/dreams_fig2b.png, as(png) replace
graph export ../../results/dreams_fig2b.eps, as(eps) replace

** figure 2c: Heatmap of two-period location choices by ability and self-confidence (grid of 36 values)
use "`aggregates'", clear
* add 0.25 to each category so that they add up to 1
foreach var of varlist s_s s_b b_s b_b {
	replace sh_`var'_rel2avg = sh_`var'_rel2avg + 0.25
}
keep qt_abil qt_self sh_s_s_rel2avg sh_s_b_rel2avg sh_b_s_rel2avg sh_b_b_rel2avg
rename sh_s_s_rel2avg ss_1
rename sh_s_b_rel2avg sb_2
rename sh_b_s_rel2avg bs_3
rename sh_b_b_rel2avg bb_4
reshape long ss_ sb_ bs_ bb_, i(qt_abil qt_self) j(count)
egen z = rmin(ss_ sb_ bs_ bb_)
* Create 6 values for ability (3 terciles * SS/SB)
gen new_abil = qt_abil * 10 + count if count == 1 | count == 3
replace new_abil = new_abil[_n-1] if missing(new_abil)
sort new_abil
egen tag = tag(new_abil)
replace tag = . if tag == 0
egen abil = seq() if !missing(tag)
replace abil = abil[_n-1] if missing(abil)
drop new_abil tag
* Create 6 values for self-esteem (3 terciles * BS/BB)
sort qt_abil qt_self count
gen new_self = qt_self * 10 + 1 if count == 1 | count == 3
replace new_self = new_self[_n-1] + 1 if missing(new_self)
sort new_self
egen tag = tag(new_self)
replace tag = . if tag == 0
egen self = seq() if !missing(tag)
replace self = self[_n-1] if missing(self)
* Rstrict to 36 grid values
sort qt_abil qt_self count
keep qt_abil qt_self z abil self
save "`grid36'", replace

* Note: contour in stata does not graph 36 equally-sized squares
* Create artificial categories to plot squares of the same size
clear
set obs 2
gen self = 0.5 in 1
replace self = 6.5 in 2
forvalues x = 0(1)7 {
	gen abil`x' = `x'
}
reshape long abil, i(self) j(count)
replace abil = 0.5 if abil == 0
replace abil = 6.5 if abil == 7
drop count
save "`tmp_self'", replace

clear
set obs 2
gen abil = 0.5 in 1
replace abil = 6.5 in 2
forvalues x = 1(1)6 {
	gen self`x' = `x'
}
reshape long self, i(abil) j(count)
drop count
* At this point, grid has 64 values (8x8) instead of 36 (6x6)
append using "`tmp_self'"
sort abil self
append using "`grid36'"
sort abil self
* Assign same shares of two-period locations to artificial cases (borders)
replace z = z[_n+1] if self == 0.5 & abil >= 1 & abil <= 6
replace z = z[_n-1] if self == 6.5 & abil >= 1 & abil <= 6
save "`grid64'", replace
* Fill information in artificial cases (borders)
gen temp_abil = abil
replace temp_abil = . if temp_abil != 1 & temp_abil != 6
replace temp_abil = 0.5 if temp_abil == 1
replace temp_abil = 6.5 if temp_abil == 6
gen temp_self = self
keep if !missing(temp_abil)
keep qt_abil - temp_self
rename temp_* *
merge 1:1 abil self using "`grid64'", update
drop qt_abil qt_self _merge
order abil self
sort abil self
grstyle set graphsize 10cm 14cm
twoway contour z abil self, heatmap levels(100) crule(intensity) scolor(240 255 255) ecolor(0 20 30) zlabel(0.18(0.02)0.34, format(%3.2fc) labsize(small)) xlabel(1.5 "1" 3.5 "2"  5.5 "3", format(%12.0fc) notick) ylabel(1.5 "1" 3.5 "2"  5.5 "3", format(%12.0fc) notick angle(horizontal)) ztitle("") ytitle("Ability tercile") xtitle("Self-confidence tercile") aspect(1) xscale(lstyle(none)) yscale(lstyle(none))
graph export ../../results/dreams_fig2c.png, as(png) replace
graph export ../../results/dreams_fig2c.eps, as(eps) replace
grstyle set graphsize 10cm 10cm
