/*-----------------------------------------------------------------------
dreams_table5.do

Stata do file to replicate table 5 of the article
'City of dreams',
by Jorge De la Roca, Gianmarco Ottaviano and Diego Puga,
Journal of the European Economic Association
-----------------------------------------------------------------------*/

* initialise
clear
clear matrix
set more off
set varabbrev off

* define locals and tempfiles
local controls_earnings "exp_big exp_big_exp exp exp2 tenure tenure2 i.occupation2d i.sector2d_90 i.cbsa_g"
tempfile tmp_panel
tempfile tmp_cbsa
tempfile tmp_fe
tempfile it_xb

* read panel
use ../../data/processed/nlsy_panel_sample, clear

** generate extra variables
sort person_id year
* include only working episodes after junior period
gen m_age = age if junior_period == 1
egen min_age = min(m_age), by (person_id)
drop if missing(min_age)
drop if age < min_age
* big-city indicator
gen big = .
replace big = 1 if cbsa_pop2010 >= $size_big & !missing(cbsa_pop2010)
replace big = 0 if cbsa_pop2010 < $size_big | missing(cbsa_pop2010)
* big-city experience
* (if location missing assign experience to previous known location)
replace big = big[_n-1] if missing(fips) & person_id == person_id[_n-1]
by person_id: gen exp_big = sum(wkswk_li) if big == 1
replace exp_big = 0 if missing(exp_big)
replace exp_big = exp_big[_n-1] if exp_big == 0 & person_id == person_id[_n-1]
* convert experience and tenure from weeks to years
rename experience exp
replace exp_big = exp_big / 52.18
* experience and tenure squared
foreach var of varlist exp tenure {
	gen `var'2 = `var' * `var'
}
* experience interactions
gen exp_big_exp = exp_big * exp

** additional sample restrictions
* keep only working episodes
keep if lfstatus == 1
* keep workers in private sector (for profit firms)
keep if worker_type == 1

** make aggregates of similarly-sized cities for location fixed effects to be computed on the basis of enough individuals
preserve
use ../../data/src/cbsa/metro_1980_2010, clear
rename (metroid m_t_a_10) (cbsa pop2010)
gen cbsa_g = cbsa
replace cbsa_g = 990100 if pop2010 < 100000
forvalues i = 100(25)500 {
	replace cbsa_g = 990`i' if pop2010 > (`i' - 25) * 1000 & pop2010 <= `i' * 1000
}
forvalues i = 550(50)1000 {
	replace cbsa_g = 990`i' if pop2010 > (`i' - 50) * 1000 & pop2010 <= `i' * 1000
}
forvalues i = 1100(100)2000 {
	replace cbsa_g = 990`i' if pop2010 > (`i' - 100) * 1000 & pop2010 <= `i' * 1000
}
egen cbsa_g_pop = mean(pop2010), by (cbsa_g)
keep cbsa cbsa_g cbsa_g_pop
save "`tmp_cbsa'", replace

** combine with nlsy panel
restore
merge m:1 cbsa using "`tmp_cbsa'", keep(master match) nogen
* estimate worker fe using entire work histories (incuding spells in non-cbsa areas)
egen cbsa_ind = max(cbsa_g), by (person_id)
replace cbsa_g = 1 if missing(cbsa_g) & !missing(cbsa_ind)
drop cbsa_ind

* initialise fe interactions for loop
gen fe_exp = .
gen fe_exp2 = .
gen fe_exp_big = .
gen fe_exp_big_exp = .
gen ln_wage_fe = .
save "`tmp_panel'", replace

** compute initial set of fixed effects without fe * experience interactions
xtset person_id year
xtreg ln_wage `controls_earnings', fe cluster(person_id)
xtset, clear
* save fe
predict xb
gen resid = ln_wage - xb
egen fe = mean(resid), by (person_id)
drop xb resid
collapse (mean) fe, by (person_id)
save "`tmp_fe'", replace

** full fixed-effects specification
* initialise for loop
local itno = 0
local ssd_coef = 100
local rmse = 100
local d_rmse = 100
matrix COEFF = J(512,1,10^6)
* iterate until convergence
while `ssd_coef' >= 1 | `d_rmse' >= 1 {
	quiet {
		local itno = `itno' + 1
		use "`tmp_panel'", clear
		* recover previous set of fixed-effects
		merge m:1 person_id using "`tmp_fe'", nogen
		* update interactions of estimated fixed-effects and experience variables
		replace fe_exp = fe * exp
		replace fe_exp2 = fe * exp2
		replace fe_exp_big = fe * exp_big
		replace ln_wage_fe = ln_wage - fe
		* estimate the value of such interactions and all other coefficients using pooled ols
		reg ln_wage_fe fe_exp fe_exp2 fe_exp_big `controls_earnings', vce(cluster person_id)
		local rmse_1 = e(rmse)
		matrix COEFFN = e(b)'
		* predict dependent variable excluding interactions between estimated fixed-effects and experience variables
		preserve
		foreach var of varlist fe_exp fe_exp2 fe_exp_big {
			replace `var' = 0
		}
		predict xb
		keep person_id year xb
		sort person_id year
		save "`it_xb'", replace
		restore
		merge 1:1 person_id year using "`it_xb'", keepusing(xb) nogen
		* we re-estimate fixed effects but now addressing heterogeneity in treatment effect
		gen double fe_numerator = ln_wage - xb
		gen double fe_denominator = 1 + _b[fe_exp]*exp + _b[fe_exp2]*exp2 + _b[fe_exp_big]*exp_big
		replace fe_denominator = 0.001 if fe_denominator == 0
		gen double fe_new = fe_numerator / fe_denominator
		collapse (mean) fe=fe_new, by (person_id)
		save "`tmp_fe'", replace
		* check for convergence in estimated coefficients between previous and current iteration
		clear
		set obs 1
		svmat double COEFF
		svmat double COEFFN
		gen double sd_coef = ((COEFFN1 - COEFF1)^2)*10^6
		egen ssdc = sum(sd_coef)
		matrix drop COEFF
		matrix rename COEFFN COEFF
		local ssd_coef "ssdc"
		local d_rmse = (`rmse' - `rmse_1')*10^7
		local rmse = `rmse_1'
	}
	display "Iteration `itno'"
	display "SSD Coef.:" `ssd_coef'
	display "D RMSE: " `d_rmse'
	display ""
}
* combine worker fe and nlsy panel
use "`tmp_fe'", clear
rename fe fe_c1
label var fe_c1 "Worker fixed-effects in earnings regression (col. 1)"
save "`tmp_fe'", replace
use "`tmp_panel'", clear
merge m:1 person_id using "`tmp_fe'", keepusing(fe_c1) keep(match) nogen
* adjust percentiles relative to median
replace afqt_pctl = afqt_pctl / 100
replace self_pctl = self_pctl / 100
egen med_afqt = median(afqt_pctl)
egen med_self = median(self_pctl)
replace afqt_pctl = afqt_pctl - med_afqt
replace self_pctl = self_pctl - med_self
* experience interactions
gen exp_self = exp * self_pctl
gen exp2_self = exp2 * self_pctl
gen exp_afqt = exp * afqt_pctl
gen exp2_afqt = exp2 * afqt_pctl
* big-city experience interactions
gen exp_big_afqt = exp_big * afqt_pctl
gen exp_big_self = exp_big * self_pctl
gen exp_big_afqt_exp = exp_big_exp * afqt_pctl
gen exp_big_self_exp = exp_big_exp * self_pctl
* self-confidence and ability in big city
gen big_self = big * self_pctl
gen big_afqt = big * afqt_pctl
* initialise next fe estimation
gen fe = fe_c1
save "`tmp_panel'", replace

** full fixed-effects specification, column (3) with exp_big_self added
* initialise for loop
local itno = 0
local ssd_coef = 100
local rmse = 100
local d_rmse = 100
matrix COEFF = J(512,1,10^6)
* iterate until convergence
while `ssd_coef' >= 1 | `d_rmse' >= 1 {
	quiet {
		local itno = `itno' + 1
		use "`tmp_panel'", clear
		* recover previous set of fixed-effects
		merge m:1 person_id using "`tmp_fe'", nogen
		* update interactions of estimated fixed-effects and experience variables
		replace fe_exp = fe * exp
		replace fe_exp2 = fe * exp2
		replace fe_exp_big = fe * exp_big
		replace ln_wage_fe = ln_wage - fe
		* estimate the value of such interactions and all other coefficients using pooled ols
		reg ln_wage_fe fe_exp fe_exp2 fe_exp_big `controls_earnings' exp_big_self, vce(cluster person_id)
		local rmse_1 = e(rmse)
		matrix COEFFN = e(b)'
		* predict dependent variable excluding interactions between estimated fixed-effects and experience variables
		preserve
		foreach var of varlist fe_exp fe_exp2 fe_exp_big {
			replace `var' = 0
		}
		predict xb
		keep person_id year xb
		sort person_id year
		save "`it_xb'", replace
		restore
		merge 1:1 person_id year using "`it_xb'", keepusing(xb) nogen
		* we re-estimate fixed effects but now addressing heterogeneity in treatment effect
		gen double fe_numerator = ln_wage - xb
		gen double fe_denominator = 1 + _b[fe_exp]*exp + _b[fe_exp2]*exp2 + _b[fe_exp_big]*exp_big
		replace fe_denominator = 0.001 if fe_denominator == 0
		gen double fe_new = fe_numerator / fe_denominator
		collapse (mean) fe=fe_new, by (person_id)
		save "`tmp_fe'", replace
		* check for convergence in estimated coefficients between previous and current iteration
		clear
		set obs 1
		svmat double COEFF
		svmat double COEFFN
		gen double sd_coef = ((COEFFN1 - COEFF1)^2)*10^6
		egen ssdc = sum(sd_coef)
		matrix drop COEFF
		matrix rename COEFFN COEFF
		local ssd_coef "ssdc"
		local d_rmse = (`rmse' - `rmse_1')*10^7
		local rmse = `rmse_1'
	}
	display "Iteration `itno'"
	display "SSD Coef.:" `ssd_coef'
	display "D RMSE: " `d_rmse'
	display ""
}
* combine worker fe and nlsy panel
use "`tmp_fe'", clear
rename fe fe_c3
label var fe_c3 "Worker fixed-effects in earnings regression (col. 3)"
save "`tmp_fe'", replace
use "`tmp_panel'", clear
drop fe
merge m:1 person_id using "`tmp_fe'", keepusing(fe_c3) keep(match) nogen
save "`tmp_panel'", replace

** table
* keep same obs throughout
drop if missing(fe_c1) | missing(exp)
* separate indicator for non-cbsa areas to keep obs there
replace cbsa_g = 1 if missing(cbsa_g)

* label variables
label var exp_big "Big-city experience"
label var exp_big_exp "Big-city exp. $\times$ experience"
label var fe_exp_big "Big-city exp. $\times$ worker fixed-effect"
label var exp_big_afqt "Big-city exp. $\times$ ability"
label var exp_big_self "Big-city exp. $\times$ self-confidence"
label var big_afqt "Big city $\times$ ability"
label var big_self "Big city $\times$ self-confidence"
label var exp "Experience"
label var exp2 "Experience$^2$"
label var fe_exp "Experience $\times$ worker fixed-effect"
label var fe_exp2 "Experience$^2$ $\times$ worker fixed-effect"
label var exp_afqt "Experience $\times$ ability"
label var exp2_afqt "Experience$^2$ $\times$ ability"
label var exp_self "Experience $\times$ self-confidence"
label var exp2_self "Experience$^2$ $\times$ self-confidence"
label var tenure "Tenure"
label var tenure2 "Tenure$^2$"
* define controls
local tablecontrols "exp_big exp_big_exp fe_exp_big exp_big_afqt exp_big_self big_afqt big_self exp exp2 fe_exp fe_exp2 exp_afqt exp2_afqt exp_self exp2_self tenure tenure2"
local city_size "exp_big exp_big_exp"
local fe "fe_exp_big fe_exp fe_exp2"
local afqt "exp_big_afqt exp_afqt exp2_afqt"
local self "exp_big_self exp_self exp2_self"
local intercepts "big_self big_afqt"
local lf_controls "exp exp2 tenure tenure2"
local indicators "i.cbsa_g i.occupation2d i.sector2d_90"

* column (1), fe
gen fe = fe_c1
replace fe_exp = exp * fe
replace fe_exp2 = exp2 * fe
replace fe_exp_big = exp_big * fe
replace fe_exp_big_exp = exp_big_exp * fe
xtset person_id year
xtreg ln_wage `city_size' `fe' `lf_controls' `indicators', fe cluster (person_id)
estimates store log5_a

* column (2), ability
xtreg ln_wage `city_size' `afqt' `lf_controls' `indicators', fe cluster (person_id)
estimates store log5_b

* column (3), fe + self-confidence
replace fe = fe_c3
replace fe_exp = exp * fe
replace fe_exp2 = exp2 * fe
replace fe_exp_big = exp_big * fe
replace fe_exp_big_exp = fe * exp_big_exp
xtreg ln_wage `city_size' `fe' `self' `lf_controls' `indicators', fe cluster (person_id)
estimates store log5_c

* column (4), ability + self-confidence
xtreg ln_wage `city_size' `afqt' `self' `lf_controls' `indicators', fe cluster (person_id)
estimates store log5_d

* column (5), ability + self-confidence + intercepts
xtreg ln_wage `city_size' `afqt' `self' `intercepts' `lf_controls' `indicators', fe cluster (person_id)
estimates store log5_e

* write latex file
estadd local icity "Yes" : log5_*
estadd local isector "Yes" : log5_*
estadd local wfe "Yes" : log5_*
estout log5_* using "../../results/dreams_table5.tex", keep(`tablecontrols') order(`tablecontrols') cells(b(star fmt(%9.4f)) se(par fmt(%9.4f))) starlevels($^{*}$ 0.10 $^{**}$ 0.05 $^{***}$ 0.01) substitute( "(" "\raisebox{.7ex}[0pt]{\scriptsize (" ")" ")}") label collabels(,none) eqlabels(,none) mlabels(,none) nonumbers noomitted stats(icity isector wfe N r2, labels("City/city group indicators" "2-digit occupation \& sector indicators" "Worker fixed effects" "Observations" "\$R^2$") fmt(%9.0g %9.0g %9.0g %9.0fc %9.4f)) style(tex) replace
