/*-----------------------------------------------------------------------
nlsy_panel.do

Stata do file used to recreate the NLSY79 panel for
'City of dreams',
by Jorge De la Roca, Gianmarco Ottaviano and Diego Puga,
Journal of the European Economic Association

Constructs NLSY79 panel from source files
-----------------------------------------------------------------------*/

* initialise
clear
set more off
set varabbrev off

* define tempfiles
tempfile tmp
tempfile tmp_fips
tempfile tmp_panel

** read nlsy source files

* read raw nlsy data, using dictionary downloaded from nlsy site
infile using ../../data/src/nlsy/raw_data.dct, clear
* use downloaded do-file to assign label values
do ../../data/src/nlsy/raw_data-value-labels

** demographics
rename (R0000100 R0173600 R0214700 R0214800) (person_id sample_type race sex)
replace sex = 0 if sex == 2
gen birth_year = 1900 + R0410300 if R0410300 > 0
replace birth_year = 1900 + R0000500 if missing(birth_year)
gen birth_month = R0410100 if R0410100 > 0
replace birth_month = R0000300 if missing(birth_month)
keep if sample_type <= 8

** read geocode nlsy data if available
if ("$NLSYDisableGeocode" != "1") {
	preserve
	* read geocoded nlsy data for locations prior to 1979
	clear
	infile using ../../data/src/nlsy/geocode/survey_and_created_variables.dct
	* use provided do-file to assign label values
	do ../../data/src/nlsy/geocode/survey_and_created_variables-value-labels
	* fips code: state and county of residence at birth
	replace R0001000 = . if R0001000 <= 0
	replace R0001000 = 100 + R0001000
	gen state = string(R0001000)
	gen state_birth = substr(state,2,2)
	replace R0000900 = . if R0000900 <= 0
	replace R0000900 = 1000 + R0000900
	gen county = string(R0000900)
	gen county_birth = substr(county,2,3)
	gen fips_birth = state_birth + county_birth
	gen fips_birth_raw = fips_birth
	gen length = length(fips_birth) if fips_birth != ""
	replace fips_birth = "" if length < 5
	replace fips_birth_raw = "" if length == 3
	drop state county length
	* fips code: state and county of residence at 14
	replace R0001600 = . if R0001600 <= 0
	replace R0001600 = 100 + R0001600
	gen state = string(R0001600)
	gen state_at14 = substr(state,2,2)
	replace R0001500 = . if R0001500 <= 0
	replace R0001500 = 1000 + R0001500
	gen county = string(R0001500)
	gen county_at14 = substr(county,2,3)
	gen fips_at14 = state_at14 + county_at14
	gen fips_at14_raw = fips_at14
	gen length = length(fips_at14) if fips_at14 != ""
	replace fips_at14 = "" if length < 5
	replace fips_at14_raw = "" if length == 3
	keep R0000100 fips_birth fips_at14
	rename R0000100 person_id
	save `tmp', replace
	restore
	merge 1:1 person_id using `tmp', keep(match) nogen
	preserve
	* read geocode nlsy data for survey years
	clear
	infile using ../../data/src/nlsy/geocode/location.dct
	* use provided do-file to assign label values
	do ../../data/src/nlsy/geocode/location-value-labels
	* keep state and county of residence
	keep R0000100 R0219001 R0219002 R0408001 R0408002 R0648001 R0648002 R0899001 R0899002 R1150000 R1150001 R1523000 R1523001 R1893000 R1893001 R2260000 R2260001 R2450000 R2450001 R2882500 R2882600 R3076900 R3077000 R3411300 R3411400 R3659500 R3659600 R4009500 R4009600 R4420600 R4420700 R5090500 R5090600 R5176300 R5176400 R6489500 R6489600 R7017300 R7017400 R7714800 R7714900 R8507600 R8507700 T0999300 T0999400 T2221100 T2221200 T3119100 T3119200
	rename * c_*
	rename c_R0000100 person_id
	rename c_* c_#, renumber
	forvalues x = 1(2)47 {
		order c_`x', last
	}
	* string variables for states
	foreach var of varlist c_2 - c_48 {
		replace `var' = . if `var' <= 0
		replace `var' = 100 + `var'
		gen s`var' = string(`var')
		gen st_s`var' = substr(s`var',2,2)
	}
	* string variables for counties
	foreach var of varlist c_1 - c_47 {
		replace `var' = . if `var' <= 0
		replace `var' = 1000 + `var'
		gen c`var' = string(`var')
		gen ct_c`var' = substr(c`var',2,3)
	}
	drop sc* cc*
	* create fips codes combining states and counties for each year
	local i = 1
	local j = 1
	local k = 2
	while `i' <= 24 {
		while `j' <= 47 {
			while `k' <= 48 {
				gen fips_`i' = st_sc_`k' + ct_cc_`j'
				local i = `i' + 1
				local j = `j' + 2
				local k = `k' + 2
			}
		}
	}
	* keep only valid fips codes (5 digits)
	foreach var of varlist fips_1 - fips_24 {
		gen l_`var' = length(`var') if `var' != ""
		replace `var' = "" if l_`var' < 5
	}
	* prepare data set to merge
	keep person_id fips*
	save `tmp', replace
	restore
	merge 1:1 person_id using `tmp', keep(match) nogen
	destring fips*, replace
	}
else {
	gen fips = .
}

** non-interview status
gen yr_non_int_1 = 0
local i = 2
while `i' <= 25 {
	foreach var of varlist R0406310 R0618810 R0898510 R1144710 R1519910 R1890300 R2257400 R2444600 R2870100 R3073900 R3400500 R3655900 R4006400 R4417500 R5080500 R5165800 R6478500 R7006300 R7703500 R8495900 T0987500 T2209800 T3107600 T4112100 {
		replace `var' = . if `var' == -4
		replace `var' = 1 if `var' > 0 & !missing(`var')
		replace `var' = 0 if missing(`var')
		rename `var' yr_non_int_`i'
		local i = `i' + 1
	}
}

** education, achievement tests and self-confidence
* educational enrollment
local i = 1
	while `i' <= 22 {
		foreach var of varlist R0216601 R0406501 R0619001 R0898301 R1145101 R1520301 R1891001 R2258101 R2445501 R2871201 R3074901 R3401601 R3657001 R4007501 R4418601 R5104000 R5166902 R6479700 R7007400 R7704700 R8497100 T0988900 {
		rename `var' yr_educ_enrolled_`i'
		replace yr_educ_enrolled_`i' = . if yr_educ_enrolled_`i' < 0
		local i = `i' + 1
	}
}
gen yr_educ_enrolled_23 = .
gen yr_educ_enrolled_24 = .
gen yr_educ_enrolled_25 = .
* highest grade of education completed
local i = 1
while `i' <= 25 {
	foreach var of varlist R0216701 R0406401 R0618901 R0898201 R1145001 R1520201 R1890901 R2258001 R2445401 R2871101 R3074801 R3401501 R3656901 R4007401 R4418501 R5103900 R5166901 R6479600 R7007300 R7704600 R8497000 T0988800 T2210700 T3108600 T4113000 {
		rename `var' yr_educ_highest_`i'
		replace yr_educ_highest_`i' = . if yr_educ_highest_`i' < 0
		local i = `i' + 1
	}
}
* afqt
rename R0618301 afqt_80
replace afqt_80 = . if afqt_80 < 0
* self-confidence
local i = 1
while `i' <= 10 {
	foreach var of varlist R0303500 R0303600 R0303700 R0303800 R0303900 R0304000 R0304100 R0304200 R0304300 R0304400 {
		rename `var' self_80_item`i'
		replace self_80_item`i' = . if self_80_item`i' <= 0
		local i = `i' + 1
	}
}
rename (R0304410 R0304420 R0304430 R0304440) (self_80_score self_80_irt self_80_zscore self_80_zpctl)
replace self_80_score = . if self_80_score <= 0
replace self_80_irt = . if self_80_irt <= 0
replace self_80_zscore = . if self_80_zscore <= 0
replace self_80_zpctl = . if self_80_zpctl <= 0
drop self_80_irt self_80_zscore self_80_item*

** demographics: marital status, children, working spouse and parents' education
* marital status
local i = 1
while `i' <= 25 {
	foreach var of varlist R0217500 R0405600 R0618600 R0898400 R1144900 R1520100 R1890800 R2257900 R2445300 R2870900 R3074600 R3401300 R3656700 R4007200 R4418300 R5081300 R5166600 R6479200 R7006900 R7704200 R8496600 T0988400 T2210400 T3108300 T4112800 {
		rename `var' yr_marital_`i'
		replace yr_marital_`i' = . if yr_marital_`i' < 0
		local i = `i' + 1
	}
}
* number of children
local i = 1
	while `i' <= 25 {
		foreach var of varlist R0218001 R0407601 R0647101 R0898838 R1146830 R1522037 R1892737 R2259837 R2448037 R2877600 R3076842 R3407700 R3659047 R4009447 R4444700 R5087500 R5172800 R6486400 R6839600 R7548600 R8255400 T0857100 T2015100 T2998600 T3942500 {
		rename `var' yr_children_`i'
		replace yr_children_`i' = . if yr_children_`i' < 0
		local i = `i' + 1
	}
}
* age of youngest child in household
local i = 1
	while `i' <= 24 {
		foreach var of varlist R0218002 R0407602 R0647102 R0898839 R1146831 R1522038 R1892738 R2259838 R2448038 R2877700 R3076843 R3407800 R3659048 R4009448 R4444800 R5087600 R5172900 R6486500 R7014300 R7711900 R8504400 T0996100 T2217900 T3115900 {
		rename `var' yr_agechildren_`i'
		replace yr_agechildren_`i' = . if yr_agechildren_`i' < 0
		local i = `i' + 1
	}
}
* annual weeks of spouse's work (last calendar year)
gen yr_spouse_wkswk_1 = .
local i = 2
while `i' <= 25 {
	foreach var of varlist R0226100 R0413700 R0659600 R0902900 R1202700 R1602000 R1902000 R2302000 R2503100 R2903100 R3104500 R3504500 R3704500 R4123200 R4519200 R5218200 R5817700 R6749700 R7396900 R8090200 T0321700 T1471300 T2520200 T3506500 {
		rename `var' yr_spouse_wkswk_`i'
		replace yr_spouse_wkswk_`i' = . if yr_spouse_wkswk_`i' < 0
		local i = `i' + 1
	}
}
* number of hours worked per week by spouse's (last calendar year)
gen yr_spouse_hrswk_1 = .
local i = 2
while `i' <= 25 {
	foreach var of varlist R0226200 R0413800 R0659700 R0903000 R1202800 R1602100 R1902100 R2302100 R2503200 R2903200 R3104600 R3504600 R3704600 R4123300 R4519300 R5218300 R5817800 R6749800 R7397000 R8090300 T0321800 T1471400 T2520300 T3506600 {
		rename `var' yr_spouse_hrswk_`i'
		replace yr_spouse_hrswk_`i' = . if yr_spouse_hrswk_`i' < 0
		local i = `i' + 1
	}
}
* parents' education
rename (R0006500 R0007900) (educ_mother educ_father)
replace educ_mother = . if educ_mother < 0
replace educ_father = . if educ_father < 0

** employment status, earnings, tenure and job type
** note: main/most recent job is always job #1 in 1979 and since 1993
* labor force status (same year)
local i = 1
while `i' <= 18 {
	foreach var of varlist R0214900 R0406300 R0618800 R0898500 R1144700 R1519900 R1890600 R2257700 R2445100 R2870600 R3074300 R3401000 R3656400 R4006900 R4418000 R5081000 R5166300 R6478900 {
		rename `var' yr_lfstatus_`i'
		replace yr_lfstatus_`i' = . if yr_lfstatus_`i' <= 0
		local i = `i' + 1
	}
}
rename T0988100 yr_lfstatus_22
replace yr_lfstatus_22 = . if yr_lfstatus_22 <= 0
gen yr_lfstatus_19 = .
gen yr_lfstatus_20 = .
gen yr_lfstatus_21 = .
gen yr_lfstatus_23 = .
gen yr_lfstatus_24 = .
gen yr_lfstatus_25 = .
recode yr_lfstatus_16 4=3 5=7
recode yr_lfstatus_17 4=3 5=7
recode yr_lfstatus_18 4=3 5=7
recode yr_lfstatus_22 4=3 5=7
* hourly wages main/most recent job, cps definition (same year)
local i = 1
while `i' <= 25 {
	foreach var of varlist R0047010 R0263710 R0446810 R0702510 R0945610 R1256010 R1650810 R1923410 R2318210 R2526010 R2925010 R3127800 R3523500 R3728500 R4416800 R5079800 R5165200 R6478000 R7005700 R7702900 R8495200 T0986800 T2209100 T3106900 T4110900 {
		rename `var' yr_wage_`i'
		replace yr_wage_`i' = . if yr_wage_`i' <= 0
		local i = `i' + 1
	}
}
* weeks of tenure in job #1  (same year)
local i = 1
while `i' <= 25 {
	foreach var of varlist R0068710 R0333221 R0539410 R0833810 R1081010 R1456710 R1803510 R2165110 R2372510 R2763410 R3005210 R3332610 R3597610 R3947800 R4416300 R5079300 R5164600 R6477500 R7005200 R7702400 R8494700 T0986300 T2208600 T3106400 T4110400 {
		rename `var' yr_tenure_`i'
		replace yr_tenure_`i' = . if yr_tenure_`i' < 0
		local i = `i' + 1
	}
}
* worker (job) type in main/most recent job, cps definition (same year)
local i = 1
while `i' <= 25 {
	foreach var of varlist R0046800 R0263500 R0446600 R0702300 R0945400 R1255800 R1650600 R1923200 R2318000 R2525800 R2924800 R3127500 R3523200 R3728200 R4182300 R4587905 R5270700 R6473100 R6592300 R7210100 R7898500 T1037500 T1298500 T3186500 T3309200 {
		rename `var' yr_worker_type_`i'
		replace yr_worker_type_`i' = . if yr_worker_type_`i' <= 0
		local i = `i' + 1
	}
}
forvalues x = 16(1)25 {
	recode yr_worker_type_`x' 1=0 2=1 3=6 4=3
	recode yr_worker_type_`x' 0=2
}

** occupation and industry codes
* type of occupation, main/most recent job, 1970 census 3-digit (same year)
local i = 1
while `i' <= 19 {
	foreach var of varlist R0046400 R0263400 R0446400 R0702100 R0945000 R1255400 R1650200 R1922800 R2317600 R2525400 R2924400 R3127100 R3522800 R3727800 R4182100 R4587904 R5270600 R6472600 R6591800 {
		rename `var' yr_occ70_`i'
		replace yr_occ70_`i' = . if yr_occ70_`i' <= 0
		local i = `i' + 1
	}
}
* type of occupation, main/most recent job, 1980 census 3-digit (same year)
local i = 4
while `i' <= 19 {
	foreach var of varlist R0828000 R0945300 R1255700 R1650500 R1923100 R2317900 R2525700 R2924700 R3127400 R3523100 R3728100 R4193700 R4587901 R5270900 R6473700 R6592900 {
		rename `var' yr_occ80_`i'
		replace yr_occ80_`i' = . if yr_occ80_`i' <= 0
		local i = `i' + 1
	}
}
* type of occupation, main/most recent job, 2000 census 3-digit (same year)
local i = 20
while `i' <= 25 {
	foreach var of varlist R7209600 R7898000 T0138400 T1298000 T2326500 T3308700 {
		rename `var' yr_occ00_`i'
		replace yr_occ00_`i' = . if yr_occ00_`i' <= 0
		local i = `i' + 1
	}
}
forvalues x = 21(1)25 {
	replace yr_occ00_`x' = int(yr_occ00_`x' / 10)
}
* type of business or industry, main/most recent job, 1970 census 3-digit (same year)
local i = 1
while `i' <= 19 {
	foreach var of varlist R0046300 R0263300 R0446300 R0702000 R0944900 R1255300 R1650100 R1922700 R2317500 R2525300 R2924300 R3127000 R3522700 R3727700 R4182000 R4587903 R5270500 R6472100 R6591300 {
		rename `var' yr_sector70_`i'
		replace yr_sector70_`i' = . if yr_sector70_`i' <= 0
		local i = `i' + 1
	}
}
* type of business or industry, main/most recent job, 2000 census 3-digit (same year)
local i = 20
while `i' <= 25 {
	foreach var of varlist R7209100 R7897500 T0137900 T1297500 T2326000 T3308200 {
		rename `var' yr_sector00_`i'
		replace yr_sector00_`i' = . if yr_sector00_`i' <= 0
		local i = `i' + 1
	}
}
forvalues x = 21(1)25 {
	replace yr_sector00_`x' = int(yr_sector00_`x'/10)
}

** labor market experience and time spent unemployed or out of labor force
* weeks worked since last interview
local i = 1
while `i' <= 25 {
	foreach var of varlist R0215300 R0406700 R0645800 R0897300 R1145800 R1521000 R1891700 R2258800 R2446200 R2872000 R3075700 R3402400 R3657800 R4008300 R4419400 R5082400 R5167700 R6480500 R7008200 R7705500 R8497900 T0989700 T2211500 T3109400 T4113900 {
		rename `var' yr_wkswk_li_`i'
		replace yr_wkswk_li_`i' = . if yr_wkswk_li_`i' < 0
		local i = `i' + 1
	}
}
* weeks spent unemployed since last interview
local i = 1
while `i' <= 25 {
	foreach var of varlist R0215400 R0406900 R0645900 R0897400 R1145900 R1521100 R1891800 R2258900 R2446300 R2872100 R3075800 R3402500 R3657900 R4008400 R4419500 R5082500 R5167800 R6480600 R7008300 R7705600 R8498000 T0989800 T2211600 T3109500 T4114000 {
		rename `var' yr_wksunem_li_`i'
		replace yr_wksunem_li_`i' = . if yr_wksunem_li_`i' < 0
		local i = `i' + 1
	}
}
* weeks spent out of the labor force since last interview
local i = 1
while `i' <= 25 {
	foreach var of varlist R0215500 R0407000 R0646000 R0897500 R1146000 R1521200 R1891900 R2259000 R2446400 R2872200 R3075900 R3402600 R3658000 R4008500 R4419600 R5082600 R5167900 R6480700 R7008400 R7705700 R8498100 T0989900 T2211700 T3109600 T4114100 {
		rename `var' yr_wksolf_li_`i'
		replace yr_wksolf_li_`i' = . if yr_wksolf_li_`i' < 0
		local i = `i' + 1
	}
}

** risk measure
gen risk_scale = T3094800 if T3094800 > 0
replace risk_scale = T4093900 if T4093900 > 0 & missing(risk_scale)

** keep relevant variables
keep person_id sample_type race sex educ* birth_* afqt* self* yr_* fips* risk*
rename (*_1 *_2 *_3 *_4 *_5 *_6 *_7 *_8 *_9 *_10 *_11 *_12 *_13 *_14 *_15 *_16 *_17 *_18 *_19 *_20 *_21 *_22 *_23 *_24 *_25) (*_1979 *_1980 *_1981 *_1982 *_1983 *_1984 *_1985 *_1986 *_1987 *_1988 *_1989 *_1990 *_1991 *_1992 *_1993 *_1994 *_1996 *_1998 *_2000 *_2002 *_2004 *_2006 *_2008 *_2010 *_2012)

** reshape data into an individual-year panel data set
reshape long fips_ yr_non_int_ yr_educ_enrolled_ yr_educ_highest_ yr_lfstatus_ yr_wage_ yr_tenure_ yr_worker_type_ yr_occ70_ yr_occ80_ yr_occ00_ yr_sector70_ yr_sector00_ yr_wkswk_li_ yr_wksunem_li_ yr_wksolf_li_ yr_spouse_wkswk_ yr_spouse_hrswk_ yr_marital_ yr_children_ yr_agechildren_, i(person_id) j(year)
rename yr_*_ *
format non_int sector00 %9.0g

** demographics
gen any_children = 0
replace any_children = 1 if sex == 0 & children > 0 & !missing(children)
replace any_children = 1 if sex == 0 & children == 0 & !missing(agechildren)
replace any_children = 1 if sex == 1 & children > 0 & year < 2000
replace any_children = 1 if sex == 1 & !missing(agechildren) & year > 2000


** standardize occupation categories (using David Dorn's crosswalks)
* transform 2000 categories into 1990 categories (data available since 2002)
rename occ00 occ
merge m:1 occ using ../../data/src/nlsy/cpi_ind_occ/occ2000_occ1990dd, keep(master match) nogen
drop occ
rename occ1990dd occ1990_2000
* transform 1980 categories to 1990 categories (data available from 1982 to 2000)
rename occ80 occ
merge m:1 occ using ../../data/src/nlsy/cpi_ind_occ/occ1980_occ1990dd, keep(master match) nogen
drop occ
rename occ1990dd occ1990_1980
* transform 1970 categories to 1990 categories (data available from 1979 to 2000)
rename occ70 occ
merge m:1 occ using ../../data/src/nlsy/cpi_ind_occ/occ1970_occ1990dd, keep(master match) nogen
drop occ
rename occ1990dd occ1990_1970
* create occupation variable with consistent categories over time
sort person_id year
gen occupation3d = occ1990_2000
replace occupation3d = occ1990_1980 if missing(occupation3d)
replace occupation3d = occ1990_1970 if missing(occupation3d)
drop occ1990_2000 occ1990_1980 occ1990_1970
* 2-digit occupation categories
gen occupation2d = int(occupation3d/10)

** standardize sector categories (ipums consistent long-term classification, 1990 basis)
* 1970 classification to 1990 consistent classification
preserve
import excel ../../data/src/nlsy/cpi_ind_occ/ind1990_tt.xls, cellrange(A12:V374) clear
replace A = "" if A == "#"
destring A, replace
keep A J
rename (A J) (ind90_70 sector70)
drop if missing(sector70)
tempfile ind90_70
save `ind90_70', replace
restore
merge m:1 sector70 using `ind90_70', keep(master match) nogen
* 2000 classification to 1990 consistent classification
preserve
import excel ../../data/src/nlsy/cpi_ind_occ/ind1990_tt.xls, cellrange(A12:V374) clear
replace A = "" if A == "#"
destring A, replace
keep A Q
rename (A Q) (ind90_00 sector00)
drop if missing(sector00)
tempfile ind90_00
save `ind90_00', replace
restore
merge m:1 sector00 using `ind90_00', keep(master match) nogen
* 3-digit sector categories, time-consistent classification
sort person_id year
gen sector3d_90 = ind90_70 if year <= 2000
replace sector3d_90 = ind90_00 if year > 2000
* 2-digit sector categories
gen sector2d_90 = int(sector3d_90/10)
drop sector70 sector00 ind90_*

** adjust nominal values for inflation (cpi 1982-1984 = 100)
* source: https://data.bls.gov/timeseries/CUSR0000SA0
preserve
import excel ../../data/src/nlsy/cpi_ind_occ/cpi_sa_1947_2015.xlsx, cellrange(A12) clear
rename (A M) (year cpi)
keep if !missing(year)
keep year cpi
save `tmp', replace
restore
merge m:1 year using `tmp', keep(master match) nogen
* adjust wages for inflation
* wages are in cents of dollars, cpi 1982-1984 = 100
* dividing by cpi converts nominal cents into real dollars
replace wage = wage / cpi
* code as missings wages that exceed 250 dollars per hour
replace wage = . if wage > 250
drop cpi

* users without access to the geocode nlsy79 data that wish to check
* that the replication code runs smoothly can set
* global NLSYGenerateFakeLocations = 0 in _dreams_run.do;
* this will randomly generate a fake location history for each respondent
* that will allow the code to run but generate meaningless results with
* the same format but different values than the actual results in the article;
* the fake location history will be created whenever "$NLSYDisableGeocode" == "1"
* but only kept if "$NLSYGenerateFakeLocations" == "1"
if ("$NLSYDisableGeocode" == "1") {
	drop fips fips_
	* possible values for fake fips codes are all in any cbsa
	preserve
	import excel "../../data/src/cbsa/CBSA_def2009.xls", cellrange(A5:L1866) clear
	rename K fips
	keep fips
	destring fips, replace
	drop if missing(fips)
	sort fips
	gen fips_order = _n
	count
	local fips_n = `r(N)'
	save `tmp_fips', replace
	restore
	* randomly assign one of the potential fips codes as fips_birth to each individual
	gen fips_order = runiformint(1,`fips_n')
	sort person_id year
	replace fips_order = fips_order[_n-1] if person_id == person_id[_n-1]
	merge m:1 fips_order using `tmp_fips', nogen keep(match)
	rename fips fips_birth
	* keep fips_birth as fips_at14 for 61% of individuals, others get a random fips
	gen fips_order2 = runiformint(1,`fips_n')
	sort person_id year
	replace fips_order2 = fips_order2[_n-1] if person_id == person_id[_n-1]
	gen rmigrant = runiform(0,100)
	replace rmigrant = rmigrant > 61
	sort person_id year
	replace rmigrant = rmigrant[_n-1] if person_id == person_id[_n-1]
	replace fips_order = fips_order2 if rmigrant == 1
	merge m:1 fips_order using `tmp_fips', nogen keep(match)
	rename fips fips_at14
	drop fips_order2 rmigrant
	* keep fips_at14 as fips in 1979 for 79.2% of individuals, others get a random fips
	gen fips_order2 = runiformint(1,`fips_n')
	sort person_id year
	replace fips_order2 = fips_order2[_n-1] if person_id == person_id[_n-1]
	gen rmigrant = runiform(0,100)
	replace rmigrant = rmigrant > 79.2
	sort person_id year
	replace rmigrant = rmigrant[_n-1] if person_id == person_id[_n-1]
	replace fips_order = fips_order2 if rmigrant == 1
	merge m:1 fips_order using `tmp_fips', nogen keep(match)
	rename fips fips_
	drop fips_order2 rmigrant
	* keep fips in 1979 as fips every year for 52.9% of individuals, others get a random fips from a random year
	gen fips_order2 = runiformint(1,`fips_n')
	sort person_id year
	replace fips_order2 = fips_order2[_n-1] if person_id == person_id[_n-1]
	gen rmigrant = runiform(0,100)
	replace rmigrant = rmigrant > 52.9
	sort person_id year
	replace rmigrant = rmigrant[_n-1] if person_id == person_id[_n-1]
	gen ryear = runiformint(1980,1996)
	sort person_id year
	replace ryear = ryear[_n-1] if person_id == person_id[_n-1]
	replace fips_order = fips_order2 if rmigrant == 1 & year >= ryear
	merge m:1 fips_order using `tmp_fips', nogen keep(match)
	replace fips_ = fips
	drop fips fips_order fips_order2 rmigrant ryear
	sort person_id year
}

** fix county code changes and misclassifications
rename fips_ fips

* county changes
* based on David Dorn's county code changes
* https://www.ddorn.net/data/FIPS_County_Code_Changes.pdf
foreach var of varlist fips_birth fips_at14 fips {
	* 1981: Cibola county, NM (FIPS 35006) is created out of parts of Valencia county, NM (FIPS 35061)
	replace `var' = 35061 if `var' == 35006
	* 1981: La Paz county, AZ (FIPS 4012) is created out of parts of Yuma county, AZ (FIPS 4027)
	replace `var' =  4027 if `var' ==  4012
	* 1983: Washabaugh county, SD (FIPS 46131) merges into Jackson county, SD (FIPS 46071)
	replace `var' = 46071 if `var' == 46131
	* 1995: The independent city of South Boston, VA (FIPS 51780) merges into Halifax county, VA (FIPS 51083)
	replace `var' = 51083 if `var' == 51780
	* 1997: Yellowstone National Park territory, MT (FIPS 30113) is merged into Gallantin, MT (FIPS 30031) and Park, MT (FIPS 30067) counties
	replace `var' = 30031 if `var' == 30113
	* 1997: Dade county, FL (FIPS 12025) is renamed as Miami-Dade county, FL (FIPS 12086)
	replace `var' = 12086 if `var' == 12025
	* 2001: The independent city of Clifton Forge, VA (FIPS 51560) merges into Alleghany county, VA (FIPS 51005)
	replace `var' = 51005 if `var' == 51560
	* 2001: Broomfield county, CO (FIPS 8014) is created out of parts of Adams, Boulder, Jefferson, and Weld counties
	* Most of the population came from Boulder
	replace `var' =  8013 if `var' ==  8014
	* Post sample changes
	* 2013: Bedford (independent) city, VA (FIPS 51515) was changed to town status and added to Bedford County (FIPS 51019)
	* 2015: Shannon County, SD (FIPS 46113) is renamed to Oglala Lakota County, SD (FIPS 46102)
}

* Attachment 102: Federal Information Processing Standards Publication 5-1
* https://www.nlsinfo.org/content/cohorts/nlsy79/other-documentation/geocode-codebook-supplement/attachment-102-state-fips
* Ohio State University Center for Human Resource Research (CHRR)
* Note from CHRR: In addition to the FIPS county codes, the following codes were used when the place name was not specific enough or the place name could not be found
foreach var of varlist fips_birth fips_at14 fips {
	* Replace New York City, NY (36500) by Kings County, NY (36047), the most populous county in NYC
	replace `var' = 36047 if `var' == 36500
	* Replace Long Island, NY (36600) by Suffolk County, NY (36103)
	replace `var' = 36103 if `var' == 36600
	* Replace Attica, NY (36300) by Wyoming County, NY (36121)
	replace `var' = 36121 if `var' == 36300
	* Replace Tulsa, OK (40400) by Tulsa County, OK (40143)
	replace `var' = 40143 if `var' == 40400
	* Replace Milwaukee, WI (55600) by Milwaukee County, WI (55079)
	replace `var' = 55079 if `var' == 55600
	* Replace Kansas City, MO (29700) by Jackson County, MO (29095)
	replace `var' = 29095 if `var' == 29700
	* Replace Richardson, TX (48701) by Dallas County, TX (48113)
	replace `var' = 48113 if `var' == 48701
	* Replace Amarillo, TX (48700) by Potter County, TX (48375)
	replace `var' = 48113 if `var' == 48375
	* Replace Pleasant Grove, TX (48701) by Dallas County, TX (48113)
	replace `var' = 48113 if `var' == 48703
	* Replace Huntington, WV (54200) by Cabell County, WV (54011)
	replace `var' = 54011 if `var' == 54200
	* Replace Cut Off, LA (22200) by Lafourche Parish, LA (22057)
	replace `var' = 22057 if `var' == 22200
	* Replace Salem, OR (41200) by Marion County, OR (41047)
	replace `var' = 41047 if `var' == 41200
	* Replace Laton Hill, AL (1200) by Washington County, AL (1129)
	replace `var' = 1129 if `var' == 1200

	* Alaska Historical Codes (1979-2008)
	* Fairbanks North Star Borough changed its code from 02818 to 02090
	replace `var' = 2090 if `var' == 2818
	* Barrow Division (02040) its part of North Slope Borough (02185)
	replace `var' = 2185 if `var' == 2040
	* Seward Division (02210) its part of Kenai Peninsula Borough (2122)
	replace `var' = 2122 if `var' == 2210
	* Valdez-Chitina-Whittier Division (02260) its part of Valdez-Cordova Census Area (02261)
	replace `var' = 2261 if `var' == 2260
}

** sample restrictions
* drop non-interview waves
drop if non_int == 1

** create cumulative variables
* age
gen age = year - birth_year
* labor market experience
sort person_id year
by person_id: gen experience = sum(wkswk_li) if age >= 18
* experience in big cities is calculated later as it depends on a partiuclar city-size threshold
* time in unemployment and out of the labor force
by person_id: gen unemployment = sum(wksunem_li) if age >= 18
by person_id: gen outlf = sum(wksolf_li) if age >= 18
drop wksunem_li wksolf_li
* express experience in years
replace experience = experience / 52.18
* express tenure in years
replace tenure = tenure / 52.18

** order and label variables
order person_id year non_int birth* age sex race sample_type educ_enrolled educ_highest educ_* marital spouse_wkswk spouse_hrswk any_children children agechildren lfstatus wkswk_li worker_type wage tenure experience unemployment outlf occupation* sector* afqt* self_80_score self_80_zpctl risk_scale fips_birth fips_at14 fips

label define sex 0 "Female" 1 "Male"
label val sex sex
label define educ_enrolled 1 "Not enrolled, completed less than high school" 2 "Enrolled, high school" 3 "Enrolled, educ_college" 4 "Not enrolled, completed high school"
label val educ_enrolled educ_enrolled
label define lfstatus 1 "Working" 2 "With job, not at work" 3 "Unemployed" 4 "Keeping house" 5 "Going to school" 6 "Unable to work" 7 "Other" 8 "In active forces"
label val lfstatus lfstatus
label define worker_type 1 "Private for profit company" 2 "Government" 3 "Self-employed" 4 "Without pay" 5 "Working in family business" 6 "Non-profit organization"
label val worker_type worker_type
compress

label var person_id "Individual identifier"
label var year "Year"
label var non_int "Non-interview indicator"
label var birth_year "Birth year"
label var birth_month "Birth month"
label var age "Age"
label var sex "Sex"
label var race "Race"
label var sample_type "Sample classification"
label var educ_enrolled "Educational enrollment status"
label var educ_highest "Highest education attained at survey date"
label var educ_mother "Mother's years of education"
label var educ_father "Father's years of education"
label var marital "Marital status"
label var spouse_wkswk "Number of weeks worked by spouse in past calendar year"
label var spouse_hrswk "Number of hours worked per week by spouse in past calendar year"
label var any_children "Has at least one child"
label var children "\# of children (since 2000, only for female respondents and only bio children)"
label var agechildren "Age of youngest child (only individuals with children)"
label var lfstatus "Labor force status (generated for selected years)"
label var wkswk_li "Weeks worked since last interview"
label var worker_type "Class of worker (job type), CPS definition"
label var wage "Hourly real wage, main job, dollars, CPS definition (1982-84 = 100)"
label var tenure "Tenure with employer (years)"
label var experience "Cumulative experience (years)"
label var unemployment "Cumulative unemployment spells (weeks)"
label var outlf "Cumulative out-of-labor-force spells (weeks)"
label var occupation3d "Occupation, CPS definition, 1990 Census 3-digit (standardized)"
label var occupation2d "Occupation, CPS definition, 1990 Census 2-digit (standardized)"
label var sector3d_90 "3-digit sector (IPUMS consistent long-term classification, 1990 basis)"
label var sector2d_90 "2-digit sector (IPUMS consistent long-term classification, 1990 basis)"
label var afqt_80 "AFQT percentile (full sample), 1980 (revised in 2006)"
label var self_80_score "Self-confidence, raw score, 1980"
label var self_80_zpctl "Self-confidence, weighted z score percentile (full sample), 1980"
label var risk_scale "Risk aversion (10 = fully prepared to take risks)"
label var fips_birth "Residence FIPS code of birth county"
label var fips_at14 "Residence FIPS code at age 14"
label var fips "Residence FIPS code"
save `tmp_panel', replace

** generate labor force status from weekly array data for selected years

* use dictionary automatically downloaded from nlsy
clear
infile using ../../data/src/nlsy/lfstatus_data.dct
* use program automatically downloaded to assign label values
do ../../data/src/nlsy/lfstatus_data-value-labels

* assign a week (1-52) to each interview date
foreach var of varlist R6963300 - T3195602 {
	replace `var' = . if `var' <= 0
}
rename (R6963300 R6963301 R6963302) (d_2000 m_2000 y_2000)
rename (R7656300 R7656301 R7656302) (d_2002 m_2002 y_2002)
rename (R7800500 R7800501 R7800502) (d_2004 m_2004 y_2004)
rename (T1200700 T1200701 T1200702) (d_2008 m_2008 y_2008)
rename (T2260600 T2260601 T2260602) (d_2010 m_2010 y_2010)
rename (T3195600 T3195601 T3195602) (d_2012 m_2012 y_2012)

* convert months into days
foreach var of varlist m_2000 m_2002 m_2004 m_2008 m_2010 m_2012 {
	replace `var' = 0 if `var' == 1
	replace `var' = 31 if `var' == 2
	replace `var' = 31 + 28 if `var' == 3
	replace `var' = 31 + 28 + 31 if `var' == 4
	replace `var' = 31 + 28 + 31 + 30 if `var' == 5
	replace `var' = 31 + 28 + 31 + 30 + 31 if `var' == 6
	replace `var' = 31 + 28 + 31 + 30 + 31 + 30 if `var' == 7
	replace `var' = 31 + 28 + 31 + 30 + 31 + 30 + 31 if `var' == 8
	replace `var' = 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 if `var' == 9
	replace `var' = 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 if `var' == 10
	replace `var' = 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 if `var' == 11
	replace `var' = 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30 if `var' == 12
}

* create weeks for each month-day combination
foreach num of numlist 2000 2002 2004 2008 2010 2012 {
	gen date_`num' = int((m_`num' + d_`num')/7) + 1
	replace date_`num' = 52 if date_`num' == 53
}

* rename weekly arrays to year-week terms
* 2000
local i = 1
	while `i' <= 52 {
	foreach var of varlist W0880200 - W0885300 {
		rename `var' w_2000_`i'
		local i = `i' + 1
		}
	}
* 2002
local i = 1
	while `i' <= 52 {
	foreach var of varlist W0937300 - W0942400 {
		rename `var' w_2002_`i'
		local i = `i' + 1
		}
	}
* 2004
local i = 1
	while `i' <= 52 {
	foreach var of varlist W1002800 - W1008000 {
		rename `var' w_2004_`i'
		local i = `i' + 1
		}
	}
* 2008
local i = 1
	while `i' <= 52 {
	foreach var of varlist W1119600 - W1124700 {
		rename `var' w_2008_`i'
		local i = `i' + 1
		}
	}
* 2010
local i = 1
	while `i' <= 52 {
	foreach var of varlist W1177500 - W1182600 {
		rename `var' w_2010_`i'
		local i = `i' + 1
		}
	}
* 2012
local i = 1
	while `i' <= 52 {
	foreach var of varlist W1253200 - W1258300 {
		rename `var' w_2012_`i'
		local i = `i' + 1
		}
	}

** Create labor force status in selected years
gen lfstatus_2000 = .
local i = 1
	while `i' <= 52 {
	foreach var of varlist w_2000_1 - w_2000_52 {
		replace lfstatus_2000 = `var' if date_2000 == `i'
		local i = `i' + 1
		}
	}
gen lfstatus_2002 = .
local i = 1
	while `i' <= 52 {
	foreach var of varlist w_2002_1 - w_2002_52 {
		replace lfstatus_2002 = `var' if date_2002 == `i'
		local i = `i' + 1
		}
	}
gen lfstatus_2004 = .
local i = 1
	while `i' <= 52 {
	foreach var of varlist w_2004_1 - w_2004_52 {
		replace lfstatus_2004 = `var' if date_2004 == `i'
		local i = `i' + 1
		}
	}
gen lfstatus_2008 = .
local i = 1
	while `i' <= 52 {
	foreach var of varlist w_2008_1 - w_2008_52 {
		replace lfstatus_2008 = `var' if date_2008 == `i'
		local i = `i' + 1
		}
	}
gen lfstatus_2010 = .
local i = 1
	while `i' <= 52 {
	foreach var of varlist w_2010_1 - w_2010_52 {
		replace lfstatus_2010 = `var' if date_2010 == `i'
		local i = `i' + 1
		}
	}
gen lfstatus_2012 = .
local i = 1
	while `i' <= 52 {
	foreach var of varlist w_2012_1 - w_2012_52 {
		replace lfstatus_2012 = `var' if date_2012 == `i'
		local i = `i' + 1
		}
	}
foreach num of numlist 2000 2002 2004 2008 2010 2012 {
	replace lfstatus_`num' = 1 if lfstatus_`num' >= 100 & !missing(lfstatus_`num')
}

* reshape labur force status data into an individual-year level data set
rename R0000100 person_id
keep person_id lfstatus_2000 - lfstatus_2012
reshape long lfstatus_, i(person_id) j(year)
rename lfstatus_ lfstatus_gen
recode lfstatus_gen 0=. 2=7 4=3 7=8 5=7

** merge with nlsy79 panel data set
merge 1:1 person_id year using `tmp_panel', keep(using match) nogen
replace lfstatus = lfstatus_gen if missing(lfstatus) & (year == 2000 | year == 2002 | year == 2004 | year == 2008 | year == 2010 | year == 2012)
drop lfstatus_gen
sort person_id year

* label labor force status
label define lfst 1 "Working" 2 "With job, not at work" 3 "Unemployed" 4 "Keeping house" 5 "Going to school" 6 "Unable to work" 7 "Other, OLF" 8 "In active forces"
label val lfstatus lfst
label var person_id "Individual identifier"
label var year "Year"
label var lfstatus "Labor force status (generated for selected years)"

* if a fake location history was generated ("$NLSYDisableGeocode" == "1"),
* keep it only if "$NLSYGenerateFakeLocations" == "1"
if ("$NLSYDisableGeocode" == "1" & "$NLSYGenerateFakeLocations" != "1") {
	drop fips_birth fips_at14 fips
}

* save panel
compress
label data "'City of dreams' De la Roca, Ottaviano & Puga. NLSY Panel"
sort person_id year
save ../../data/processed/nlsy_panel, replace
describe, fullnames
